package comparator._2_accessor;

import static comparator._2_accessor.AccessorComparator.invert;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.List;

import junit.framework.Assert;

import org.junit.Test;

import comparator.test.common.bean.Customer;

/**
 * JUnit4-Testcase fr den verbesserten
 * {@link Comparator} {@link AccessorComparator}.
 * 
 * @author Heiner K&uuml;cker
 */
public class AccessorComparatorTest
{
    /**
     * Test method for {@link AccessorComparator#compare}.
     */
    @SuppressWarnings( "static-method" )
    @Test
    public final void testCompareSimple()
    {
        final Comparator<Customer> comparator =
                new AccessorComparator<>(
                        Customer.SUR_NAME_GETTER );

        final List<Customer> customerList =
                createCustomerList();

        Collections.sort(
                customerList ,
                comparator );

        Assert.assertEquals( "Meier" , customerList.get( 0 ).getSurName() );
        Assert.assertEquals( "Meyer" , customerList.get( 1 ).getSurName() );
    }

    /**
     * Test Sortieren nach mehreren Properties.
     * 
     * Test method for {@link java.util.Comparator#compare}.
     */
    @SuppressWarnings( "static-method" )
    @Test
    public final void testCompareMulti()
    {
        final Comparator<Customer> comparator1 =
                new AccessorComparator<>(
                        Customer.SUR_NAME_GETTER ,
                        Customer.PRE_NAME_GETTER );
                
        final List<Customer> customerList =
                createCustomerList();

        Collections.sort(
                customerList ,
                comparator1 );

        Assert.assertEquals( "Meier" , customerList.get( 0 ).getSurName() );
        Assert.assertEquals( "Meyer" , customerList.get( 1 ).getSurName() );

        // Sortieren nach mehreren Propertys
        final Comparator<Customer> comparator2 =
                new AccessorComparator<>(
                        Customer.PRE_NAME_GETTER ,
                        Customer.SUR_NAME_GETTER );

        Collections.sort(
                customerList ,
                comparator2 );

        Assert.assertEquals( "Anton" , customerList.get( 0 ).getPreName() );
        Assert.assertEquals( "Berta" , customerList.get( 1 ).getPreName() );
    }

    /**
     * Test null-Wert sortieren, null ist kleiner als nicht-null.
     * 
     * Test method for {@link java.util.Comparator#compare}.
     */
    @SuppressWarnings( "static-method" )
    @Test
    public final void testCompareNull()
    {
        final Comparator<Customer> comparator =
                new AccessorComparator<>(
                        Customer.SUR_NAME_GETTER );

        final List<Customer> customerList =
                createCustomerList();

        // Setzen der Property, nach welcher zu sortieren ist, auf null
        customerList.get( 1 ).setSurName( null );

        Collections.sort(
                customerList ,
                comparator );

        Assert.assertEquals( null , customerList.get( 0 ).getSurName() );
    }

    /**
     * Test absteigend sortieren.
     * 
     * Test method for {@link java.util.Comparator#compare}.
     */
    @SuppressWarnings( "static-method" )
    @Test
    public final void testCompareInverted()
    {
        final Comparator<Customer> comparator =
                new AccessorComparator<>(
                        invert( Customer.SUR_NAME_GETTER ) );
                
        final List<Customer> customerList =
                createCustomerList();

        Collections.sort(
                customerList ,
                comparator );

        Assert.assertEquals( "Meyer" , customerList.get( 0 ).getSurName() );
        Assert.assertEquals( "Meier" , customerList.get( 1 ).getSurName() );
    }

    /**
     * Test alles zusammen.
     * 
     * Test method for {@link java.util.Comparator#compare}.
     */
    @SuppressWarnings( "static-method" )
    @Test
    public final void testCompareMultiInvertedNull()
    {
        final List<Customer> customerList =
                createCustomerList();

        {
            final Comparator<Customer> comparator =
                    new AccessorComparator<>(
                            invert( Customer.SUR_NAME_GETTER ) ,
                            Customer.PRE_NAME_GETTER ,
                            Customer.BIRTH_GETTER );

            Collections.sort(
                    customerList ,
                    comparator );

            Assert.assertEquals( "Meyer" , customerList.get( 0 ).getSurName() );
            Assert.assertEquals( "Meier" , customerList.get( 1 ).getSurName() );
        }

        {
            final Comparator<Customer> comparator =
                    new AccessorComparator<>(
                            Customer.PRE_NAME_GETTER ,
                            invert( Customer.SUR_NAME_GETTER ) ,
                            Customer.BIRTH_GETTER );

            Collections.sort(
                    customerList ,
                    comparator );

            Assert.assertEquals( "Anton" , customerList.get( 0 ).getPreName() );
            Assert.assertEquals( "Berta" , customerList.get( 1 ).getPreName() );
        }

        {
            final Comparator<Customer> comparator =
                    new AccessorComparator<>(
                            Customer.BIRTH_GETTER ,
                            invert( Customer.SUR_NAME_GETTER ) ,
                            Customer.PRE_NAME_GETTER );

            // Setzen der Property, nach welcher zu sortieren ist, auf null
            customerList.get( 1 ).setBirth( null );

            Collections.sort(
                    customerList ,
                    comparator );

            Assert.assertEquals( null , customerList.get( 0 ).getBirth() );
        }
    }


    /**
     * Erzeugen Testdaten.
     * 
     * @return Liste mit {@link Customer}-Objekten
     */
    private static List<Customer> createCustomerList()
    {
        final List<Customer> customerList = new ArrayList<>();

        {
            final Customer customer1 =
                    new Customer();

            customer1.setPreName( "Anton" );
            customer1.setSurName( "Meyer" );
            customer1.setBirth( new Date( 0 ) );

            customerList.add( customer1 );
        }

        {
            final Customer customer2 =
                    new Customer();

            customer2.setPreName( "Berta" );
            customer2.setSurName( "Meier" );
            // Initialisieren Geburts-Datum customer2 nach Geburts-Datum customer1
            customer2.setBirth( new Date( 48L * 60L * 60L * 1000L ) );

            customerList.add( customer2 );
        }

        // Liste durchmischen, damit die Unit-Tests nicht zufllig aufgrund der initialen Reihenfolge erfolgreich sind
        Collections.shuffle( customerList );

        return customerList;
    }

}
