package comparator._3_property;

import java.util.Comparator;

import comparator.util.Getter;


/**
 * Vorgefertigter {@link Comparator}
 * fr eine bestimmte Property mit
 * Hilfe eines {@link Getter}-Objekts
 * ohne Null-Behandlung.
 * 
 * @param <B> Type-Parameter, Bean-Typ
 * @author Heiner K&uuml;cker
 */
public final class PropertyComparator<B>
extends AbstractPropertyComparator<B>
{
    /**
     * Konstruktor.
     * 
     * @param getter {@link Getter}-Objekt zum Zugriff auf die Bean-Property, nach welcher zu vergleichen ist
     */
    public PropertyComparator(
            final Getter<B, ? extends Comparable<?>> getter )
    {
        super( getter );
    }

    /**
     * @see java.util.Comparator#compare
     */
    @SuppressWarnings( { "unchecked", "cast", "rawtypes" } )
    @Override
    public final int compare(
            final B bean1 ,
            final B bean2 )
    {
        return
                // weil hier zweimal das selbe Getter-Objekt verwendet wird, ist die Typ-Kompatibilitt der beiden Werte abgesichert
                ( (Comparable) this.get( bean1 ) ).compareTo(
                        ( (Comparable) this.get( bean2 ) ) );
    }

}
