package comparator._3_property;

import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import junit.framework.Assert;

import org.junit.Test;

import comparator.test.common.TestUtil;
import comparator.test.common.bean.Customer;

/**
 * JUnit4-Testcase fr den verbesserten
 * {@link Comparator} {@link PropertyComparator}.
 * 
 * @author Heiner K&uuml;cker
 */
public class PropertyComparatorTest
{
    /**
     * Test method for {@link PropertyComparator#compare}.
     */
    @SuppressWarnings( "static-method" )
    @Test
    public final void testCompareSimple()
    {
        final Comparator<Customer> comparator =
                new PropertyComparator<>(
                        Customer.SUR_NAME_GETTER );

        final List<Customer> customerList =
                TestUtil.createCustomerList();

        Collections.sort(
                customerList ,
                comparator );

        Assert.assertEquals( "Meier" , customerList.get( 0 ).getSurName() );
        Assert.assertEquals( "Meyer" , customerList.get( 1 ).getSurName() );
    }

    /**
     * Test Sortieren nach mehreren Properties.
     * 
     * Test method for {@link PropertyComparator#compare}.
     */
    @SuppressWarnings( "static-method" )
    @Test
    public final void testCompareMulti()
    {
        final Comparator<Customer> comparator1 =
                new MultiComparator<>(
                        new PropertyComparator<>(
                                Customer.SUR_NAME_GETTER ) ,
                        new PropertyComparator<>(
                                Customer.PRE_NAME_GETTER ) );

        final List<Customer> customerList =
                TestUtil.createCustomerList();

        Collections.sort(
                customerList ,
                comparator1 );

        Assert.assertEquals( "Meier" , customerList.get( 0 ).getSurName() );
        Assert.assertEquals( "Meyer" , customerList.get( 1 ).getSurName() );

        // Sortieren nach mehreren Propertys
        final Comparator<Customer> comparator2 =
                new MultiComparator<>(
                        new PropertyComparator<>(
                                Customer.PRE_NAME_GETTER ) ,
                        new PropertyComparator<>(
                                Customer.SUR_NAME_GETTER ) );

        Collections.sort(
                customerList ,
                comparator2 );

        Assert.assertEquals( "Anton" , customerList.get( 0 ).getPreName() );
        Assert.assertEquals( "Berta" , customerList.get( 1 ).getPreName() );
    }

    /**
     * Test null-Wert sortieren, null ist kleiner als nicht-null.
     * 
     * Test method for {@link NullIsLesserPropertyComparator#compare}.
     */
    @SuppressWarnings( "static-method" )
    @Test
    public final void testCompareNull()
    {
        final Comparator<Customer> comparator =
                new NullIsLesserPropertyComparator<>(
                        Customer.SUR_NAME_GETTER );

        final List<Customer> customerList =
                TestUtil.createCustomerList();

        // Setzen der Property, nach welcher zu sortieren ist, auf null
        customerList.get( 1 ).setSurName( null );

        Collections.sort(
                customerList ,
                comparator );

        Assert.assertEquals( null , customerList.get( 0 ).getSurName() );
    }

    /**
     * Test absteigend sortieren.
     * 
     * Test method for {@link PropertyComparator#compare}.
     */
    @SuppressWarnings( "static-method" )
    @Test
    public final void testCompareInverted()
    {
        final Comparator<Customer> comparator =
                Collections.reverseOrder(
                        new PropertyComparator<>(
                                Customer.SUR_NAME_GETTER ) );

        final List<Customer> customerList =
                TestUtil.createCustomerList();

        Collections.sort(
                customerList ,
                comparator );

        Assert.assertEquals( "Meyer" , customerList.get( 0 ).getSurName() );
        Assert.assertEquals( "Meier" , customerList.get( 1 ).getSurName() );
    }

    /**
     * Test alles zusammen.
     * 
     * Test method for {@link PropertyComparator#compare}
     * and {@link NullIsLesserPropertyComparator#compare}.
     */
    @SuppressWarnings( "static-method" )
    @Test
    public final void testCompareMultiInvertedNull()
    {
        final List<Customer> customerList =
                TestUtil.createCustomerList();

        {
            final Comparator<Customer> comparator =
                    new MultiComparator<>(
                            Collections.reverseOrder(
                                    new PropertyComparator<>(
                                            Customer.SUR_NAME_GETTER ) ) ,
                            new PropertyComparator<>(
                                    Customer.PRE_NAME_GETTER ) ,
                            new PropertyComparator<>(
                                    Customer.BIRTH_GETTER ) );

            Collections.sort(
                    customerList ,
                    comparator );

            Assert.assertEquals( "Meyer" , customerList.get( 0 ).getSurName() );
            Assert.assertEquals( "Meier" , customerList.get( 1 ).getSurName() );
        }

        {
            final Comparator<Customer> comparator =
                    new MultiComparator<>(
                            new PropertyComparator<>(
                                    Customer.PRE_NAME_GETTER ) ,
                            Collections.reverseOrder(
                                    new PropertyComparator<>(
                                            Customer.SUR_NAME_GETTER ) ) ,
                            new PropertyComparator<>(
                                    Customer.BIRTH_GETTER ) );

            Collections.sort(
                    customerList ,
                    comparator );

            Assert.assertEquals( "Anton" , customerList.get( 0 ).getPreName() );
            Assert.assertEquals( "Berta" , customerList.get( 1 ).getPreName() );
        }

        {
            final Comparator<Customer> comparator =
                    new MultiComparator<>(
                            new NullIsLesserPropertyComparator<>(
                                    Customer.BIRTH_GETTER  ),
                            Collections.reverseOrder(
                                    new PropertyComparator<>(
                                            Customer.SUR_NAME_GETTER ) ) ,
                            new PropertyComparator<>(
                                    Customer.PRE_NAME_GETTER ) );

            // Setzen der Property, nach welcher zu sortieren ist, auf null
            customerList.get( 1 ).setBirth( null );

            Collections.sort(
                    customerList ,
                    comparator );

            Assert.assertEquals( null , customerList.get( 0 ).getBirth() );
        }
    }

}
