package comparator._4_null_safe;

import java.util.Comparator;

/**
 * <a href="http://de.wikipedia.org/wiki/Decorator">Dekorator </a> fr
 * einen {@link Comparator}.
 * <br/>
 * Zum dekorierten {@link Comparator} wird
 * die Funktionalitt hinzugefgt, dass
 * <code>null</code>-Werte als kleiner
 * als jeder andere Wert gelten und
 * {@link NullPointerException}
 * vermieden werden.
 * 
 * @param <T> Type-Parameter, Typ der zu vergleichenden Objekte
 * @author Heiner K&uuml;cker
 */
public final class NullIsLesserComparatorDecorator<T>
implements Comparator<T>
{
    /**
     * Originaler {@link Comparator}.
     */
    private final Comparator<T> comparatorToDecorate;

    /**
     * @param comparatorToDecorate
     */
    public NullIsLesserComparatorDecorator(
            final Comparator<T> comparatorToDecorate )
    {
        if ( comparatorToDecorate == null )
        {
            throw new IllegalArgumentException(
                    "comparatorToDecorate is null" );
        }
        this.comparatorToDecorate = comparatorToDecorate;
    }

    /**
     * @see java.util.Comparator#compare
     */
    @Override
    public final int compare(
            final T o1 ,
            final T o2 )
    {
        if ( o1 == null &&
                o2 == null )
        {
            return 0;
        }
        
        if ( o1 == null )
        {
            // null gilt als kleiner
            return -1;
        }
        if ( o2 == null )
        {
            return 1;
        }

        return this.comparatorToDecorate.compare(
                o1, 
                o2 );
    }

}
