package comparator._4_null_safe;

import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import junit.framework.Assert;

import org.junit.Test;

import comparator._3_property.PropertyComparator;
import comparator.test.common.TestUtil;
import comparator.test.common.bean.Customer;

/**
 * JUnit4-Testcase fr die Klasse {@link NullIsLesserComparatorDecorator}.
 * 
 * @author Heiner K&uuml;cker
 */
public class NullIsLesserComparatorDecoratorTest
{
    /**
     * Test method for {@link NullIsLesserComparatorDecorator#compare}.
     */
	@SuppressWarnings( { "static-method" , "boxing" } )
    @Test
	public final void test()
	{
		final Comparator<Integer> comparator =
				new NullIsLesserComparatorDecorator<>(
						new Comparator<Integer>()
						{
							@Override
							public int compare(
									final Integer o1 ,
									final Integer o2 )
							{
								return o1.compareTo( o2 );
							}
						} );

		Assert.assertTrue(
				comparator.compare(
						0 ,
						1 ) < 0 );

		Assert.assertTrue(
				comparator.compare(
						1 ,
						0 ) > 0 );

		Assert.assertTrue(
				comparator.compare(
						null ,
						1 ) < 0 );

		Assert.assertTrue(
				comparator.compare(
						1 ,
						null ) > 0 );
	}

    /**
     * Test Bean selbst ist null.
     * 
     * Test method for {@link NullIsLesserComparatorDecorator#compare}.
     */
    @SuppressWarnings( "static-method" )
    @Test
    public final void testBeanNull()
    {
        final List<Customer> customerList =
                TestUtil.createCustomerList();
        
        // Setzen einer ganzen Bean auf Null
        customerList.set( 1 , null );
        
        final Comparator<Customer> comparator =
                new NullIsLesserComparatorDecorator<>(
                        new PropertyComparator<>(
                                Customer.SUR_NAME_GETTER ) );

        Collections.sort(
                customerList ,
                comparator );        

        // erste Bean sollte null sein
        Assert.assertEquals( null , customerList.get( 0 ) );
    }

}
